import pytest


def test_sles_ec2_dracut_conf(host, get_release_value, determine_architecture):
    if determine_architecture() != 'X86_64':
        pytest.skip('Only x86_64 architecture is tested.')

    needed_drivers = (
        'ena',
        'nvme',
        'nvme-core',
        'virtio',
        'virtio_scsi',
        'xen-blkfront',
        'xen-netfront'
    )
    version = get_release_value('VERSION')
    assert version

    name = get_release_value('PRETTY_NAME').lower()
    assert name

    dracut_conf = host.file('/etc/dracut.conf.d/07-aws-type-switch.conf')

    assert dracut_conf.exists
    assert dracut_conf.is_file

    for driver in needed_drivers:
        if (
            driver.startswith('virtio') and
            (
                version.startswith('15') or
                version == '12-SP5' or
                'micro' in name
            )
        ):
            continue
        assert dracut_conf.contains(driver)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof controller unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pytest import raises
from unittest.mock import patch

from img_proof.ipa_controller import collect_tests
from img_proof.ipa_controller import test_image as controller_test_image
from img_proof.ipa_exceptions import IpaControllerException
from img_proof.ipa_cloud import IpaCloud
from img_proof.ipa_gce import GCECloud


@patch('img_proof.ipa_controller.os')
def test_collect_tests_no_dirs(mock_os):
    """Test collect tests default directories do not exist."""
    mock_os.path.exists.return_value = False

    with raises(IpaControllerException):
        collect_tests(verbose=True)


@patch.object(IpaCloud, 'test_image')
def test_controller_azure_image(mock_test_image):
    mock_test_image.return_value = (0, {'results': 'data'})

    status, results = controller_test_image(
        cloud_name='azure',
        config='tests/data/config',
        distro='sles',
        image_id='another:fake:image:id',
        no_default_test_dirs=True,
        running_instance_id='fakeinstance',
        ssh_private_key_file='tests/data/ida_test',
        test_dirs='tests/data/tests',
        tests=['test_image'],
        service_account_file='tests/azure/test-sa.json'
    )

    assert status == 0


@patch.object(IpaCloud, 'test_image')
def test_controller_ec2_image(mock_test_image):
    mock_test_image.return_value = (0, {'results': 'data'})

    status, results = controller_test_image(
        cloud_name='ec2',
        config='tests/data/config',
        cloud_config='tests/ec2/.ec2utils.conf',
        distro='sles',
        image_id='another:fake:image:id',
        no_default_test_dirs=True,
        ssh_private_key_file='tests/data/ida_test',
        ssh_key_name='test-key',
        account='awstest'
    )

    assert status == 0


@patch.object(IpaCloud, 'test_image')
@patch.object(GCECloud, '_get_driver')
@patch.object(GCECloud, '_get_credentials')
@patch.object(GCECloud, '_validate_region')
def test_controller_gce_image(
    mock_validate_region,
    mock_get_creds,
    mock_get_driver,
    mock_test_image
):
    mock_test_image.return_value = (0, {'results': 'data'})

    status, results = controller_test_image(
        cloud_name='gce',
        config='tests/data/config',
        distro='sles',
        image_id='another:fake:image:id',
        no_default_test_dirs=True,
        ssh_private_key_file='tests/data/ida_test',
        service_account_file='tests/gce/service-account.json',
        region='us-west1-a'
    )

    assert status == 0


@patch.object(IpaCloud, 'test_image')
def test_controller_ssh_image(mock_test_image):
    mock_test_image.return_value = (0, {'results': 'data'})

    status, results = controller_test_image(
        cloud_name='ssh',
        config='tests/data/config',
        distro='sles',
        ip_address='another:fake:image:id',
        no_default_test_dirs=True,
        ssh_private_key_file='tests/data/ida_test',
        ssh_user='root'
    )

    assert status == 0


@patch.object(IpaCloud, 'test_image')
@patch('img_proof.ipa_oci.oci.core')
def test_controller_oci_image(mock_oci, mock_test_image):
    mock_test_image.return_value = (0, {'results': 'data'})

    status, results = controller_test_image(
        cloud_name='oci',
        config='tests/data/config',
        cloud_config='tests/oci/config',
        distro='sles',
        image_id='another:fake:image:id',
        ip_address='another:fake:image:id',
        no_default_test_dirs=True,
        ssh_private_key_file='tests/data/ida_test',
        test_dirs='tests/data/tests',
        tests=['test_image'],
        oci_user_id=(
            'ocid1.user.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        ),
        signing_key_fingerprint=(
            '00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00'
        ),
        signing_key_file='tests/oci/api_key.pem',
        tenancy=(
            'ocid1.tenancy.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        ),
        availability_domain='Omic:PHX-AD-1',
        compartment_id=(
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
    )

    assert status == 0


@patch.object(IpaCloud, 'test_image')
def test_controller_aliyun_image(mock_test_image):
    mock_test_image.return_value = (0, {'results': 'data'})

    status, results = controller_test_image(
        cloud_name='aliyun',
        config='tests/data/config',
        distro='sles',
        image_id='another:fake:image:id',
        no_default_test_dirs=True,
        ssh_private_key_file='tests/data/ida_test',
        ssh_key_name='test-key',
        access_key='1234567890',
        access_secret='0987654321',
    )

    assert status == 0



# -*- coding: utf-8 -*-

"""Utility functions."""

# Copyright (c) 2020 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import configparser
import fnmatch
import json
import logging
import os
import random
import sys
import time

import paramiko
import yaml

from binascii import hexlify
from contextlib import contextmanager
from string import ascii_lowercase
from tempfile import NamedTemporaryFile
from paramiko.ssh_exception import AuthenticationException

from img_proof.ipa_constants import SYNC_POINTS
from img_proof.ipa_exceptions import IpaSSHException, IpaUtilsException

CLIENT_CACHE = {}


def clear_cache(ip=None):
    """Clear the client cache or remove key matching the given ip."""
    if ip:
        with ignored(Exception):
            client = CLIENT_CACHE[ip]
            del CLIENT_CACHE[ip]
            client.close()
    else:
        for client in CLIENT_CACHE.values():
            with ignored(Exception):
                client.close()
        CLIENT_CACHE.clear()


def establish_ssh_connection(ip,
                             ssh_private_key_file,
                             ssh_user,
                             port,
                             attempts=5,
                             timeout=None):
    """
    Establish ssh connection and return paramiko client.

    Raises:
        IpaSSHException: If connection cannot be established
            in given number of attempts.
    """
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    while attempts:
        try:
            client.connect(
                ip,
                port=port,
                username=ssh_user,
                key_filename=ssh_private_key_file,
                timeout=timeout
            )
        except (FileNotFoundError, AuthenticationException):
            raise
        except:  # noqa: E722
            attempts -= 1
            time.sleep(10)
        else:
            return client

    raise IpaSSHException(
        'Failed to establish SSH connection to instance.'
    )


def execute_ssh_command(client, cmd):
    """
    Execute given command using paramiko.

    Returns:
        String output of cmd execution.
    Raises:
        IpaSSHException: If stderr returns a non-empty string.
    """
    try:
        stdin, stdout, stderr = client.exec_command(cmd)
        err = stderr.read()
        out = stdout.read()
        if err:
            raise IpaSSHException(out.decode() + err.decode())
    except:  # noqa: E722
        raise
    return out.decode()


def expand_test_files(test_dirs, names):
    """
    Expand the list of test files and test descriptions.

    Returns:
        List of test files and sync points.
    Raises:
        IpaUtilsException: If names is not a list.
    """
    if not isinstance(names, list):
        raise IpaUtilsException(
            'Names must be a list containing test names'
            ' and/or test descriptions.'
        )

    tests, descriptions = get_test_files(test_dirs)

    expanded_names = []
    for name in names:
        if name in descriptions:
            expanded_names += get_tests_from_description(
                name,
                descriptions
            )
        else:
            expanded_names.append(name)

    return parse_sync_points(expanded_names, tests)


def extract_archive(client, archive_path, extract_path=None):
    """
    Extract the archive in current path using the provided client.

    If extract_path is provided extract the archive there.
    """
    command = 'tar -xf {path}'.format(path=archive_path)

    if extract_path:
        command += ' -C {extract_path}'.format(extract_path=extract_path)

    out = execute_ssh_command(client, command)
    return out


def find_test_file(name, tests):
    """
    Find test file by name, given a list of tests.

    If a specific test case is appended to test
    name, split the case and append to path.

    Raises:
        IpaUtilsException: If test file not found.
    """
    try:
        test_name, test_case = name.split('::', 1)
    except ValueError:
        test_name, test_case = name, None

    path = tests.get(test_name, None)
    if not path:
        raise IpaUtilsException(
            'Test file with name: %s cannot be found.' % test_name
        )

    if test_case:
        path = ''.join([path, '::', test_case])
    return path


def generate_instance_name(name):
    """Generate a new random name for instance."""
    return '%s-%s' % (name, get_random_string(length=5))


def get_public_ssh_key(ssh_private_key_file):
    """Get SSH public key from private key file."""
    pub_key = ssh_private_key_file + '.pub'

    try:
        with open(pub_key, "rb") as key_file:
            key = key_file.read()
    except FileNotFoundError:
        raise IpaUtilsException(
            'SSH public key file: {key_path} cannot be found.'.format(
                key_path=pub_key
            )
        )

    return key


def get_config_values(config_path, section, default='default'):
    """
    Parse ini config file and return a dict of values.

    The provided section overrides any values in default section.
    """
    values = {}

    if not os.path.isfile(config_path):
        raise IpaUtilsException(
            'Config file not found: %s' % config_path
        )

    config = configparser.ConfigParser()

    try:
        config.read(config_path)
    except Exception:
        raise IpaUtilsException(
            'Config file format invalid.'
        )

    try:
        values.update(config.items(default))
    except Exception:
        pass

    try:
        values.update(config.items(section))
    except Exception:
        pass

    return values


def get_host_key_fingerprint(client):
    """Get host key fingerprint of SSH client."""
    return hexlify(
        client.get_transport().get_remote_server_key().get_fingerprint()
    )


def get_random_string(length=12):
    """Create random string of length with ascii lowercase chars."""
    return ''.join(random.choice(ascii_lowercase) for index in range(length))


def get_ssh_client(ip,
                   ssh_private_key_file,
                   ssh_user='root',
                   port=22,
                   timeout=600,
                   wait_period=10):
    """Attempt to establish and test ssh connection."""
    if CLIENT_CACHE.get(ip):
        try:
            execute_ssh_command(CLIENT_CACHE[ip], 'ls')
        except Exception:
            clear_cache(ip)
        else:
            return CLIENT_CACHE[ip]

    start = time.time()
    end = start + timeout

    client = None
    while time.time() < end:
        try:
            client = establish_ssh_connection(
                ip,
                ssh_private_key_file,
                ssh_user,
                port,
                timeout=wait_period
            )
            execute_ssh_command(client, 'ls')
        except FileNotFoundError:
            raise IpaSSHException(
                'SSH private key file {key_file} not found.'.format(
                    key_file=ssh_private_key_file
                )
            )
        except AuthenticationException:
            raise IpaSSHException(
                'Authentication failed while establishing SSH connection.'
            )
        except:  # noqa: E722
            if client:
                client.close()
            wait_period += wait_period
        else:
            CLIENT_CACHE[ip] = client
            return client

    raise IpaSSHException(
        'Attempt to establish SSH connection failed.'
    )


def get_test_files(test_dirs):
    """
    Walk all test dirs and find all tests and test descriptions.

    Returns:
        A tuple containing a dict mapping test names to full path
        and a dict mapping test descriptions to full path.
    Raises:
        IpaUtilsException: If there are multiple test files or
            test descriptions with the same name. Or, if there is a
            name overlap with a test file and test description.
    """
    tests = {}
    descriptions = {}
    for test_dir in test_dirs:
        if not os.path.exists(test_dir):
            continue

        for root, dirs, files in os.walk(test_dir):
            test_files = fnmatch.filter(files, 'test_*.py')
            description_files = fnmatch.filter(files, 'test_*.yaml')

            for test_file in test_files:
                path = os.path.join(root, test_file)
                name, ext = test_file.split('.')
                if name not in tests:
                    tests[name] = path
                else:
                    raise IpaUtilsException(
                        'Duplicate test file name found: %s, %s'
                        % (path, tests.get(name))
                    )

            for description_file in description_files:
                path = os.path.join(root, description_file)
                name, ext = description_file.split('.')
                if name in tests:
                    raise IpaUtilsException(
                        'Test description name matches test file: %s, %s'
                        % (path, tests.get(name))
                    )
                elif name not in descriptions:
                    descriptions[name] = path
                else:
                    raise IpaUtilsException(
                        'Duplicate test description file name found: %s, %s'
                        % (path, descriptions.get(name))
                    )

    return tests, descriptions


def get_tests_from_description(name,
                               descriptions,
                               parsed=None):
    """
    Recursively collect all tests in test description.

    Args:
        name (str): Yaml test description file name.
        descriptions (dict): Dict of test description name
                             (key) and absolute file paths
                             (value).
        parsed (list): List of description paths which have
                       already been parsed to prevent infinte
                       recursion.
    Returns:
        A list of expanded test files.
    """
    tests = []
    if not parsed:
        parsed = []

    description = descriptions.get(name, None)
    if not description:
        raise IpaUtilsException(
            'Test description file with name: %s cannot be located.'
            % name
        )

    if description in parsed:
        return tests

    parsed.append(description)
    test_data = get_yaml_config(description)

    if 'tests' in test_data:
        tests += test_data.get('tests')

    if 'include' in test_data:
        for description_name in test_data.get('include'):
            tests += get_tests_from_description(
                description_name,
                descriptions,
                parsed
            )

    return tests


def get_yaml_config(config_path):
    """
    Load yaml config file and return dictionary.

    Todo:
        * This will need refactoring similar to the test search.
    """
    config_path = os.path.expanduser(config_path)
    if not os.path.isfile(config_path):
        raise IpaUtilsException(
            'Config file not found: %s' % config_path
        )

    with open(config_path, 'r') as f:
        config = yaml.safe_load(f)
    return config


@contextmanager
def ignored(*exceptions):
    """Ignore the provided exception(s)."""
    try:
        yield
    except exceptions:
        pass


def parse_sync_points(names, tests):
    """
    If test is test file find full path to file.

    Otherwise leave sync point names as is.

    Returns:
        A list of test file paths and sync points.
    Examples:
        ['test_hard_reboot']
        ['/path/to/test1', '/path/to/test2']
        ['/path/to/test1', '/path/to/test2', 'test_soft_reboot']
    """
    test_files = []

    for name in names:
        if name in SYNC_POINTS:
            test_files.append(name)
        else:
            test_files.append(find_test_file(name, tests))

    return test_files


def parse_test_name(name):
    """Parse and return formatted pytest test name string."""
    test_class = None
    if '::' in name:
        try:
            path, test_class, parens, test_case = name.split('::')
        except ValueError:
            path, test_case = name.split('::')

        test_file = path.split(os.sep)[-1].replace('.py', '')
        return '::'.join(filter(None, [test_file, test_class, test_case]))
    else:
        return name


def put_file(client, source_file, destination_file):
    """
    Copy file to instance using Paramiko client connection.
    """
    try:
        sftp_client = client.open_sftp()
        sftp_client.put(source_file, destination_file)
    except Exception as error:
        raise IpaUtilsException(
            'Error copying file to instance: {0}.'.format(error)
        )
    finally:
        with ignored(Exception):
            sftp_client.close()


@contextmanager
def redirect_output(fileobj):
    """Redirect standard out to file."""
    old = sys.stdout
    sys.stdout = fileobj
    try:
        yield fileobj
    finally:
        sys.stdout = old


@contextmanager
def ssh_config(ssh_user, ssh_private_key_file):
    """Create temporary ssh config file."""
    try:
        ssh_file = NamedTemporaryFile(delete=False, mode='w+')
        ssh_file.write('Host *\n')
        ssh_file.write('    IdentityFile %s\n' % ssh_private_key_file)
        ssh_file.write('    User %s' % ssh_user)
        ssh_file.close()
        yield ssh_file.name
    finally:
        with ignored(OSError):
            os.remove(ssh_file.name)


def update_history_log(history_log,
                       clear=False,
                       description=None,
                       test_log=None):
    """
    Update the history log file with item.
    If clear flag is provided the log file is deleted.
    """
    if not test_log and not clear:
        raise IpaUtilsException(
            'A test log or clear flag must be provided.'
        )

    if clear:
        with ignored(OSError):
            os.remove(history_log)

    else:
        history_dir = os.path.dirname(history_log)
        if not os.path.isdir(history_dir):
            try:
                os.makedirs(history_dir)
            except OSError as error:
                raise IpaUtilsException(
                    'Unable to create directory: %s' % error
                )

        with open(history_log, 'a+') as f:
            # Using append mode creates file if it does not exist
            if description:
                description = '"%s"' % description

            out = '{} {}'.format(
                test_log,
                description or ''
            )
            f.write(out.strip() + '\n')


def get_logger(log_level):
    """
    Return new console logger at provided log level.
    """
    logger = logging.getLogger('img_proof')
    logger.setLevel(log_level)

    console_handler = logging.StreamHandler()
    console_handler.setLevel(log_level)
    console_handler.setFormatter(logging.Formatter('%(message)s'))

    logger.addHandler(console_handler)
    return logger


def load_json(file_path):
    """
    Load json from file and return dictionary.
    """
    with open(file_path, 'r') as json_file:
        data = json.load(json_file)

    return data


def strtobool(val: str) -> int:
    """
    This function implements a strtobool to replace distutils requirement

    Distutils module is deprecated:
    https://www.python.org/dev/peps/pep-0632/ for details.
    """
    val = val.lower()
    if val in ('y', 'yes', 't', 'true', 'on', '1'):
        return 1
    if val in ('n', 'no', 'f', 'false', 'off', '0'):
        return 0
    raise ValueError(f"Invalid truth value {val}")



import shlex


def test_sles_smt_reg(
    check_cloud_register,
    determine_provider,
    get_smt_server_name,
    get_smt_servers,
    host
):
    provider = determine_provider()

    # ensure instance registered successfully
    assert check_cloud_register()

    # ensure the correct smt/rmt ip is used
    servers = get_smt_servers(provider)
    smt_ips = [server['ip'] for server in servers]

    result = host.run(
        'cat /etc/hosts | grep %s' % get_smt_server_name(provider)
    )
    smt_ip = shlex.split(result.stdout)[0]

    assert smt_ip in smt_ips

    # ensure region hint is in log
    cloud_register_log = host.file('/var/log/cloudregister')
    assert cloud_register_log.contains(
        'INFO:Region server arguments: ?regionHint='
    )



import json
import pytest

from susepubliccloudinfoclient import infoserverrequests


@pytest.fixture()
def check_cloud_register(host):
    def f():
        # There was an API change in registerutils and we have to check
        # which version is in place on the system
        deleted_interface_import = \
            'from cloudregister.registerutils import check_registration'
        result = host.run(
            "sudo python3 -c '{0}'".format(deleted_interface_import)
        )
        # Old interface of is_registered()
        is_registered_arg = 'registerutils.get_current_smt()'
        if result.rc == 1:
            # New interface of is_registered()
            is_registered_arg = 'registerutils.get_current_smt().get_FQDN()'
        result = host.run(
            "sudo python3 -c 'from cloudregister import registerutils; "
            "print(registerutils.is_registered("
            "{0}))'".format(is_registered_arg)
        )
        output = result.stdout.strip()
        return output in ('1', 'True')
    return f


@pytest.fixture()
def check_service(host):
    def f(service_name, running=True, enabled=True):
        is_running = None
        is_enabled = None

        if host.exists('systemctl'):
            service = host.service(service_name)

            if running is not None:
                is_running = service.is_running

            if enabled is not None:
                is_enabled = service.is_enabled
        else:
            # SystemV Init
            if running is not None:
                is_running = host.run_expect(
                    [0, 1, 3],
                    'sudo /sbin/service {0} status'.format(service_name)
                ).rc == 0

            if enabled is not None:
                is_enabled = bool(host.check_output(
                    'find -L /etc/init.d/rc?.d/ -name S??{0}'.format(
                        service_name
                    ),
                ))

        return all([
            is_running == running,
            is_enabled == enabled
        ])
    return f


@pytest.fixture()
def determine_architecture(host):
    def f():
        result = host.run('uname --m')
        return result.stdout.strip().upper()
    return f


@pytest.fixture()
def determine_provider(host):
    def f():
        result = host.run('sudo dmidecode -t system')
        output = result.stdout.lower()
        if 'amazon' in output:
            provider = 'ec2'
        elif 'microsoft' in output:
            provider = 'azure'
        elif 'google' in output:
            provider = 'gce'
        else:
            raise Exception('Provider not found.')
        return provider
    return f


@pytest.fixture()
def determine_region(host):
    def f(provider):
        if provider == 'ec2':
            result = host.run(
                'curl http://169.254.169.254/latest/meta-data/placement/'
                'availability-zone'
            )
            region = result.stdout.strip()[:-1]
        elif provider == 'gce':
            result = host.run(
                'curl "http://metadata.google.internal/computeMetadata/v1/'
                'instance/zone" -H "Metadata-Flavor: Google"'
            )
            # returns zone like: us-west1-a
            region = result.stdout.strip().rsplit('/', maxsplit=1)[-1]
            # returns region: us-west1
            region = region.rsplit('-', maxsplit=1)[0]
        elif provider == 'azure':
            result = host.run(
                'curl -H Metadata:true '
                '"http://169.254.169.254/metadata/instance'
                '?api-version=2017-12-01"'
            )
            region = json.loads(result.stdout)['compute']['location']
        return region
    return f


@pytest.fixture()
def get_baseproduct(host):
    """
    Return the name of the file the 'baseproduct' link points to.
    """
    def f():
        result = host.run('readlink -f "/etc/products.d/baseproduct"')
        return result.stdout.strip()
    return f


@pytest.fixture()
def is_sles_sap(host, get_baseproduct):
    def f():
        sap_product = '/etc/products.d/SLES_SAP.prod'
        sap = host.file(sap_product)
        base_product = get_baseproduct()

        return all([
            sap.exists,
            sap.is_file,
            base_product == sap_product
        ])
    return f


@pytest.fixture()
def is_sles_sapcal(host):
    def f():
        motd = host.file('/etc/motd')
        return motd.contains('SAPCAL')
    return f


@pytest.fixture()
def get_release_value(host):
    def f(key):
        release = host.file('/etc/os-release')

        if not release.exists:
            release = host.file('/etc/SUSE-release')

        value = None
        key += '='

        for line in release.content_string.split('\n'):
            if line.startswith(key):
                value = line[len(key):].replace('"', '').replace("'", '')
                value = value.strip()
                break

        return value
    return f


@pytest.fixture()
def get_smt_server_name(host):
    def f(provider):
        return 'smt-{}.susecloud.net'.format(provider)
    return f


@pytest.fixture()
def get_smt_servers():
    def f(provider, region=None):
        if provider == 'azure':
            provider = 'microsoft'
        elif provider == 'ec2':
            provider = 'amazon'
        elif provider == 'gce':
            provider = 'google'
        else:
            raise Exception('Provider %s unknown' % provider)

        args = [provider, 'smt', 'json']

        if region:
            args.append(region)

        output = json.loads(
            infoserverrequests.get_server_data(*args)
        )

        return output['servers']
    return f


@pytest.fixture()
def install_zypper_package(host):
    def f(name):
        result = host.run('sudo zypper -n in %s' % name)
        return result.rc
    return f


@pytest.fixture()
def confirm_license_content(host):
    def f(license_dirs, license_content):
        for license_dir in license_dirs:
            lic_dir = host.file(license_dir)

            if lic_dir.exists and lic_dir.is_directory:
                lic = host.file(license_dir + 'license.txt')
                return all([
                    lic.exists,
                    lic.is_file,
                    any(lic.contains(content) for content in license_content)
                ])

        return False
    return f



import pytest


types = (
    'c5.large',
    'i3.8xlarge',
    'i3.large',
    'm5.large',
    't3.small'
)

special_regions = [
    'ap-northeast-3',
    'cn-north-1',
    'cn-northwest-1',
    'us-gov-west-1'
]

dl_time = 20


def test_sles_ec2_network(determine_region, host):
    """
    Requires an S3 bucket in each region with the following name:
        suse-download-test-{region}

    Each bucket contains the iso:
        SLE-15-Installer-DVD-x86_64-GM-DVD2.iso
    """
    result = host.run(
        'curl http://169.254.169.254/latest/meta-data/instance-type'
    )
    instance_type = result.stdout.strip()

    if instance_type not in types:
        pytest.skip(
            'Unsupported EC2 instance type: {0}.'.format(instance_type)
        )

    region = determine_region('ec2')

    if region in special_regions:
        pytest.skip(
            'Skipped special region: {0}'.format(region)
        )

    url = 'https://suse-download-test-{0}.s3.amazonaws.com/' \
          'SLE-15-Installer-DVD-x86_64-GM-DVD2.iso'.format(
              region
          )

    for i in range(3):
        dl_result = host.run(
            'curl -o /dev/null --max-time {0} --silent '
            '--write-out "%{{size_download}}|%{{http_code}}" {1}'.format(
                dl_time,
                url
            )
        )

        size, code = dl_result.stdout.strip().split('|')

        if code == '200' and size == '1214599168':
            return

    if code != '200':
        pytest.fail('Image ISO not found for region: {0}'.format(region))
    elif size != '1214599168':
        pytest.fail('Download failed. Size: {0}'.format(str(size)))



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof Red Hat distro unit tests."""

# Copyright (c) 2020 Neal Gompa. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest

from img_proof.ipa_exceptions import IpaDistroException, IpaRedHatException
from img_proof.ipa_redhat import RedHat

from unittest.mock import MagicMock, patch


def test_redhat_get_stop_ssh_cmd():
    """Test Red Hat get stop ssh cmd method."""
    redhat = RedHat()

    redhat.init_system = 'systemd'
    assert redhat.get_stop_ssh_service_cmd() == 'systemctl stop sshd.service'

    redhat.init_system = 'init'
    assert redhat.get_stop_ssh_service_cmd() == 'service sshd stop'

    redhat.init_system = 'fake'
    with pytest.raises(IpaRedHatException) as error:
        redhat.get_stop_ssh_service_cmd()
    assert str(error.value) == \
        'The init system for this Red Hat system cannot be determined.'


def test_redhat_install_package():
    """Test install package method for Red Hat distro."""
    client = MagicMock()
    redhat = RedHat()

    with patch('img_proof.ipa_utils.execute_ssh_command',
               MagicMock(return_value='')) as mocked:
        redhat.install_package(client, 'python')

    mocked.assert_called_once_with(
        client,
        "sudo sh -c 'dnf --assumeyes --nogpgcheck install python'"
    )


@patch('img_proof.ipa_distro.time')
def test_redhat_reboot(mock_time):
    """Test soft reboot method for Red Hat distro."""
    client = MagicMock()
    channel = MagicMock()
    transport = MagicMock()
    transport.open_session.return_value = channel
    client.get_transport.return_value = transport
    redhat = RedHat()
    redhat.init_system = 'systemd'

    redhat.reboot(client)

    channel.exec_command.assert_called_once_with(
        "sudo sh -c '(sleep 1 && systemctl stop sshd.service "
        "&& shutdown -r now &)' && exit"
    )


def test_redhat_reboot_exception():
    """Test soft reboot method exception for Red Hat distro."""
    client = MagicMock()
    client.get_transport.side_effect = Exception('ERROR!')
    redhat = RedHat()
    redhat.init_system = 'systemd'

    with pytest.raises(IpaDistroException):
        redhat.reboot(client)


def test_redhat_update():
    """Test update method for Red Hat distro."""
    client = MagicMock()
    redhat = RedHat()

    with patch('img_proof.ipa_utils.execute_ssh_command',
               MagicMock(return_value='Update finished!')) as mocked:
        output = redhat.update(client)

    mocked.assert_called_once_with(
        client,
        "sudo sh -c 'dnf --assumeyes makecache;dnf --assumeyes upgrade'"
    )
    assert output == 'Update finished!'


def test_redhat_update_exception():
    """Test update method exception for Red Hat distro."""
    client = MagicMock()
    redhat = RedHat()

    with patch('img_proof.ipa_utils.execute_ssh_command', MagicMock(
               side_effect=Exception('ERROR!'))) as mocked:
        pytest.raises(
            IpaDistroException,
            redhat.update,
            client
        )

    mocked.assert_called_once_with(
        client,
        "sudo sh -c 'dnf --assumeyes makecache;dnf --assumeyes upgrade'"
    )


def test_redhat_refresh():
    """Test refresh method for Red Hat distro."""
    client = MagicMock()
    redhat = RedHat()

    with patch('img_proof.ipa_utils.execute_ssh_command',
               MagicMock(return_value='Refresh finished!')) as mocked:
        output = redhat.repo_refresh(client)

    mocked.assert_called_once_with(
        client,
        "sudo sh -c 'dnf --assumeyes makecache'"
    )
    assert output == 'Refresh finished!'


def test_redhat_refresh_exception():
    """Test refresh method exception for Red Hat distro."""
    client = MagicMock()
    redhat = RedHat()

    with patch('img_proof.ipa_utils.execute_ssh_command', MagicMock(
            side_effect=Exception('ERROR!'))) as mocked:
        pytest.raises(
            IpaDistroException,
            redhat.repo_refresh,
            client
        )

    mocked.assert_called_once_with(
        client,
        "sudo sh -c 'dnf --assumeyes makecache'"
    )



def test_sles_force_new_reg(check_cloud_register, host):
    host.run("sudo sh -c ': > /var/log/cloudregister'")
    result = host.run('sudo registercloudguest --force-new')
    assert result.rc == 0
    assert check_cloud_register()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof GCE cloud unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import pytest

from img_proof.ipa_gce import GCECloud
from img_proof.ipa_exceptions import GCECloudException, IpaRetryableError

from unittest.mock import MagicMock, patch

from googleapiclient.errors import HttpError


def get_http_error(msg, status='404'):
    resp = MagicMock()
    resp.status = status

    content = {
        'error': {
            'code': int(status),
            'message': msg
        }
    }

    return HttpError(resp, json.dumps(content).encode())


class TestGCECloud(object):
    """Test GCE cloud class."""

    @patch('img_proof.ipa_gce.service_account')
    @patch.object(GCECloud, '_validate_region')
    @patch('img_proof.ipa_gce.discovery')
    def setup_method(
        self,
        method,
        mock_discovery,
        mock_validate_region,
        mock_service_account
    ):
        """Set up kwargs dict."""
        self.kwargs = {
            'config': 'tests/data/config',
            'distro_name': 'SLES',
            'image_id': 'fakeimage',
            'no_default_test_dirs': True,
            'ssh_private_key_file': 'tests/data/ida_test',
            'test_files': ['test_image'],
            'custom_args': {
                'service_account_file': 'tests/gce/service-account.json'
            }
        }

        driver = MagicMock()
        mock_discovery.build.return_value = driver

        service_account = MagicMock()
        mock_service_account.Credentials.\
            from_service_account_file.return_value = service_account

        self.cloud = GCECloud(**self.kwargs)

    def test_gce_exception_required_args(self):
        """Test an exception is raised if required args missing."""
        self.kwargs['custom_args']['service_account_file'] = None
        self.kwargs['ssh_private_key_file'] = None

        # Test service account file required
        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        assert str(error.value) == \
            'Service account file is required to connect to GCE.'

        self.kwargs['custom_args']['service_account_file'] = \
            'tests/gce/service-account.json'

        # Test ssh private key file required
        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        assert str(error.value) == \
            'SSH private key file is required to connect to instance.'

        self.kwargs['ssh_private_key_file'] = 'tests/data/ida_test'

    @patch('img_proof.ipa_gce.service_account')
    def test_gce_get_service_account_info_invalid(self, mock_service_account):
        """Test get credentials method with invalid service account."""
        self.cloud.service_account_file = \
            'tests/gce/invalid-service-account.json'

        with pytest.raises(GCECloudException) as error:
            self.cloud._get_credentials()

        msg = 'Service account JSON file is invalid for GCE. ' \
            'client_email key is expected. See getting started ' \
            'docs for information on GCE configuration.'
        assert str(error.value) == msg

    def test_gce_get_instance(self):
        """Test gce get instance method."""
        instance = MagicMock()
        instances_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = instance
        instances_obj.get.return_value = operation
        self.cloud.compute_driver.instances.return_value = instances_obj

        val = self.cloud._get_instance()

        assert val == instance

        self.cloud.running_instance_id = 'test-instance'
        instances_obj.get.side_effect = get_http_error(
            'test-instance cannot be found.'
        )

        with pytest.raises(GCECloudException) as error:
            self.cloud._get_instance()

        exc = "Unable to retrieve instance: test-instance cannot be found."
        assert str(error.value) == exc

    def test_gce_get_network(self):
        """Test GCE get network method."""
        network = MagicMock()
        networks_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = network
        networks_obj.get.return_value = operation
        self.cloud.compute_driver.networks.return_value = networks_obj

        result = self.cloud._get_network('test-network')

        assert result == network

        networks_obj.get.side_effect = get_http_error(
            'Resource test-network not found.'
        )

        msg = 'Unable to retrieve network: Resource test-network not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_network('test-network')

        assert msg == str(error.value)

    def test_gce_get_subnet(self):
        """Test GCE get subnetwork method."""
        subnetwork = MagicMock()
        subnet_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = subnetwork
        subnet_obj.get.return_value = operation
        self.cloud.compute_driver.subnetworks.return_value = subnet_obj

        self.cloud.region = 'us-west-1a'
        result = self.cloud._get_subnet('test-subnet')

        assert result == subnetwork

        subnet_obj.get.side_effect = get_http_error(
            'Resource test-subnet not found.'
        )

        msg = 'Unable to retrieve subnet: Resource test-subnet not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_subnet('test-subnet')

        assert msg == str(error.value)

    def test_gce_get_instance_type(self):
        """Test GCE get instance type method."""
        machine_type = MagicMock()
        machine_type_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = machine_type
        machine_type_obj.get.return_value = operation
        self.cloud.compute_driver.machineTypes.return_value = machine_type_obj

        result = self.cloud._get_instance_type('n1-standard-1')
        assert result == machine_type

        machine_type_obj.get.side_effect = get_http_error(
            'Resource n1-standard-1 not found.'
        )

        msg = 'Unable to retrieve instance type: ' \
              'Resource n1-standard-1 not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_instance_type('n1-standard-1')

        assert msg == str(error.value)

    def test_gce_get_image(self):
        """Test GCE get image method."""
        image = MagicMock()
        image_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = image
        image_obj.get.return_value = operation
        self.cloud.compute_driver.images.return_value = image_obj

        result = self.cloud._get_image('fake-image-20200202')
        assert result == image

        image_obj.get.side_effect = get_http_error(
            'Resource fake-image-20200202 not found.'
        )

        msg = 'Unable to retrieve image: ' \
              'Resource fake-image-20200202 not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_image('fake-image-20200202')

        assert msg == str(error.value)

    def test_gce_get_disk(self):
        """Test GCE get image method."""
        disk = MagicMock()
        disk_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = disk
        disk_obj.get.return_value = operation
        self.cloud.compute_driver.disks.return_value = disk_obj

        result = self.cloud._get_disk('disk12')
        assert result == disk

        disk_obj.get.side_effect = get_http_error(
            'Resource disk12 not found.'
        )

        msg = 'Unable to retrieve disk: ' \
              'Resource disk12 not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_disk('disk12')

        assert msg == str(error.value)

    @patch.object(GCECloud, '_get_subnet')
    def test_get_network_config(self, mock_get_subnet):
        subnet = 'projects/test/regions/us-west1/subnetworks/sub-123'
        net = 'projects/test/global/networks/network'

        mock_get_subnet.return_value = {
            'selfLink': subnet,
            'network': net
        }

        subnet_config = self.cloud._get_network_config(
            'sub-123',
            use_gvnic=True
        )

        assert subnet_config['network'] == net
        assert subnet_config['subnetwork'] == subnet

    def test_get_shielded_instance_config(self):
        si_config = self.cloud.get_shielded_instance_config()

        assert si_config['enableSecureBoot'] is False
        assert si_config['enableVtpm']
        assert si_config['enableIntegrityMonitoring']

    def test_get_instance_config(self):
        config = self.cloud.get_instance_config(
            'instance123',
            'n1-standard-1',
            [{}],
            'service-account-123@email.com',
            'image123',
            'secretkey',
            50,
            shielded_instance_config={'shielded': 'config'},
            sev_capable=True
        )

        assert 'metadata' in config
        assert 'serviceAccounts' in config
        assert 'machineType' in config
        assert 'disks' in config
        assert 'networkInterfaces' in config
        assert 'name' in config
        assert 'shieldedInstanceConfig' in config

    @patch.object(GCECloud, '_wait_on_instance')
    @patch.object(GCECloud, '_wait_on_operation')
    @patch.object(GCECloud, '_get_network')
    @patch.object(GCECloud, '_get_image')
    @patch.object(GCECloud, '_get_instance_type')
    @patch('img_proof.ipa_utils.generate_instance_name')
    def test_gce_launch_instance(
        self,
        mock_generate_instance_name,
        mock_get_instance_type,
        mock_get_image,
        mock_get_network,
        mock_wait_on_operation,
        mock_wait_on_instance
    ):
        """Test GCE launch instance method."""
        mock_generate_instance_name.return_value = 'test-instance'
        mock_get_network.return_value = {
            'selfLink': 'projects/test/global/networks/net1'
        }
        mock_get_image.return_value = {
            'selfLink': 'projects/test/global/images/img-123'
        }
        mock_get_instance_type.return_value = {
            'selfLink': 'zones/us-west1-a/machineTypes/n1-standard-1'
        }
        mock_wait_on_operation.return_value = {}

        instances_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = {'name': 'operation123'}
        instances_obj.insert.return_value = operation
        self.cloud.compute_driver.instances.return_value = instances_obj

        self.cloud.region = 'us-west1-a'

        self.cloud._launch_instance()

        assert self.cloud.running_instance_id == 'test-instance'
        assert mock_wait_on_instance.call_count == 1

        # Exception on operation

        mock_wait_on_operation.return_value = {
            'error': {
                'errors': [{
                    'code': 'QUOTA_EXCEEDED',
                    'message': 'Too many cpus.'
                }]
            }
        }

        with pytest.raises(IpaRetryableError) as error:
            self.cloud._launch_instance()

        assert 'Failed to launch instance: Too many cpus.' == str(error.value)

        # Exception on API call

        mock_wait_on_operation.return_value = {}
        instances_obj.insert.side_effect = get_http_error(
            'Invalid instance type.',
            '412'
        )

        with pytest.raises(IpaRetryableError) as error:
            self.cloud._launch_instance()

        msg = 'Failed to launch instance: Invalid instance type.'
        assert msg == str(error.value)

    @patch.object(GCECloud, '_get_disk')
    @patch.object(GCECloud, '_get_instance')
    def test_gce_set_image_id(self, mock_get_instance, mock_get_disk):
        """Test gce cloud set image id method."""
        instance = {
            'disks': [{
                'deviceName': 'disk123',
                'boot': True,
                'source': 'https://www.googleapis.com/compute/v1/projects/'
                          'test/zones/us-west1-a/disks/disk123'
            }]
        }
        disk = {
            'sourceImage': 'projects/suse/global/images/opensuse-leap-15.0'
        }
        mock_get_instance.return_value = instance
        mock_get_disk.return_value = disk

        self.cloud._set_image_id()

        assert self.cloud.image_id == 'opensuse-leap-15.0'
        assert mock_get_instance.call_count == 1
        assert mock_get_disk.call_count == 1

    def test_gce_validate_region(self):
        """Test gce cloud set image id method."""
        zones_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = None
        zones_obj.get.return_value = operation
        self.cloud.compute_driver.zones.return_value = zones_obj

        with pytest.raises(GCECloudException) as error:
            self.cloud._validate_region()

        assert str(error.value) == \
            'Zone is required for GCE cloud framework: Example: us-west1-a'

        self.cloud.region = 'fake'

        with pytest.raises(GCECloudException) as error:
            self.cloud._validate_region()

        assert str(error.value) == \
            'fake is not a valid GCE zone. Example: us-west1-a'

    @patch.object(GCECloud, '_get_instance')
    def test_gce_is_instance_running(self, mock_get_instance):
        """Test gce cloud is instance runnning method."""
        mock_get_instance.return_value = {'status': 'RUNNING'}
        assert self.cloud._is_instance_running()
        assert mock_get_instance.call_count == 1

        mock_get_instance.return_value = {'status': 'TERMINATED'}
        assert not self.cloud._is_instance_running()

    @patch.object(GCECloud, '_get_instance')
    def test_gce_set_instance_ip(self, mock_get_instance):
        """Test gce cloud set instance ip method."""
        mock_get_instance.return_value = {
            'networkInterfaces': [{'some': 'data'}]
        }

        self.cloud.running_instance_id = 'test'

        with pytest.raises(GCECloudException) as error:
            self.cloud._set_instance_ip()

        assert str(error.value) == \
            'IP address for instance: test cannot be found.'
        assert mock_get_instance.call_count == 1

        mock_get_instance.return_value = {
            'networkInterfaces': [{'networkIP': '10.0.0.0'}]
        }
        self.cloud._set_instance_ip()

        assert self.cloud.instance_ip == '10.0.0.0'

    @patch.object(GCECloud, '_wait_on_instance')
    def test_gce_start_instance(self, mock_wait_on_instance):
        """Test gce start instance method."""
        mock_wait_on_instance.return_value = None

        instances_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = None
        instances_obj.start.return_value = operation
        self.cloud.compute_driver.instances.return_value = instances_obj

        self.cloud._start_instance()

        assert instances_obj.start.call_count == 1

    @patch.object(GCECloud, '_wait_on_instance')
    def test_gce_stop_instance(self, mock_wait_on_instance):
        """Test gce stop instance method."""
        mock_wait_on_instance.return_value = None

        instances_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = None
        instances_obj.stop.return_value = operation
        self.cloud.compute_driver.instances.return_value = instances_obj

        self.cloud._stop_instance()

        assert instances_obj.stop.call_count == 1

    def test_gce_terminate_instance(self):
        """Test gce terminate instance method."""
        instances_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = None
        instances_obj.delete.return_value = operation
        self.cloud.compute_driver.instances.return_value = instances_obj

        self.cloud._terminate_instance()
        assert instances_obj.delete.call_count == 1

    def test_gce_get_console_log(self):
        """Test gce get console log method."""
        instances_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = {'content': 'some output'}
        instances_obj.getSerialPortOutput.return_value = operation
        self.cloud.compute_driver.instances.return_value = instances_obj

        self.cloud.get_console_log()
        assert instances_obj.getSerialPortOutput.call_count == 1

    @patch('img_proof.ipa_gce.time')
    def test_wait_on_operation(self, mock_time):
        self.cloud.service_account_project = 'test_project'
        self.cloud.region = 'us-west1-a'

        mock_time.sleep.return_value = None
        mock_time.time.return_value = 10

        zone_ops_obj = MagicMock()
        operation = MagicMock()
        operation.execute.return_value = {'status': 'DONE'}
        zone_ops_obj.get.return_value = operation
        self.cloud.compute_driver.zoneOperations.return_value = zone_ops_obj

        result = self.cloud._wait_on_operation('operation213')
        assert result['status'] == 'DONE'
        assert zone_ops_obj.get.call_count == 1
